package com.jiuqi.mde;

import java.util.Hashtable;
import java.util.ArrayList;

public final class MDEFieldSet
{
    private ArrayList fields;
    private Hashtable search;
    private Hashtable indexs;
    
    public MDEFieldSet() {
        this.fields = new ArrayList(128);
        this.search = new Hashtable(127);
        this.indexs = null;
    }
    
    public int size() {
        return this.fields.size();
    }
    
    public void setCapacity(final int capacity) {
        this.fields = new ArrayList(capacity);
        this.search = new Hashtable(capacity);
    }
    
    public MDEField get(final int index) {
        return this.fields.get(index);
    }
    
    public void clear() {
        this.fields.clear();
        this.search.clear();
        this.indexs = null;
    }
    
    public void add(final MDEField field) {
        this.indexs = null;
        if (!this.search.containsKey(field.name)) {
            this.fields.add(field);
            this.search.put(field.name, field);
        }
    }
    
    public void add(final int index, final MDEField field) {
        this.indexs = null;
        if (!this.search.containsKey(field.name)) {
            this.fields.add(index, field);
            this.search.put(field.name, field);
        }
    }
    
    public void add(final MDEFieldSet fields) {
        for (int i = 0; i < fields.size(); ++i) {
            this.add(fields.get(i));
        }
    }
    
    public void remove(final int index) {
        this.indexs = null;
        final MDEField field = this.fields.get(index);
        this.search.remove(field.name);
        this.fields.remove(index);
    }
    
    public void remove(final MDEField field) {
        this.indexs = null;
        this.fields.remove(field);
        this.search.remove(field.name);
    }
    
    public MDEField find(final String name) {
        return this.search.get(name);
    }
    
    public int indexOf(final MDEField field) {
        final Integer index = this.indexs.get(field.fieldName);
        return (index == null) ? -1 : index;
    }
    
    public int indexOfFieldName(final String fieldName) {
        final Integer index = this.indexs.get(fieldName);
        return (index == null) ? -1 : index;
    }
    
    public MDEField findByFieldName(final String fieldName) {
        final Integer index = this.indexs.get(fieldName);
        return (index == null) ? null : this.fields.get(index);
    }
    
    public void complete() {
        if (this.indexs == null) {
            this.sortByFieldName();
            (this.indexs = new Hashtable()).clear();
            for (int i = 0; i < this.fields.size(); ++i) {
                final MDEField field = this.fields.get(i);
                this.indexs.put(field.fieldName, new Integer(i));
            }
        }
    }
    
    private void sortByFieldName() {
        for (int i = 1; i < this.fields.size(); ++i) {
            final MDEField field = this.fields.get(i);
            int start = 0;
            int stop = i - 1;
            int flag = -1;
            while (start <= stop) {
                final int middle = start + stop >> 1;
                flag = field.fieldName.compareTo(this.fields.get(middle).fieldName);
                if (flag > 0) {
                    start = middle + 1;
                }
                else {
                    if (flag >= 0) {
                        start = middle;
                        break;
                    }
                    stop = middle - 1;
                }
            }
            if (start < i) {
                for (int k = i; k > start; --k) {
                    this.fields.set(k, this.fields.get(k - 1));
                }
                this.fields.set(start, field);
            }
        }
    }
    
    public Object clone() {
        final MDEFieldSet result = new MDEFieldSet();
        result.add(this);
        return result;
    }
}
